@@ -19,26 +19,37 @@ import Conjure
 x ^^^ 0  =  x
 _ ^^^ _  =  0
 
+-- | inc or square based on boolean argument
+--   except for 0 where the answer is swapped
+isq :: Bool -> Int -> Int
+isq False 0  =  0
+isq True  0  =  1
+isq False x  =  x + 1
+isq True  x  =  x * x
+
 main :: IO ()
 main  =  do
-  conjure "^^^" (^^^) primitives
-  conjureWith args{usePatterns = False} "^^^" (^^^) primitives
+  -- TODO: Conjure should find isq, but doesn't due to overpruning
+  conjure "isq" isq ingredients
+
+  conjure "^^^" (^^^)   ingredients
+  conjure "^^^" (^^^) $ ingredients ++ [singlePattern]
 
   -- This example is quite the degenerate case,
-  -- it takes a while to conjure even with just 2 primitives.
+  -- it takes a while to conjure even with just 2 ingredients.
   -- I am leaving it commented-out for now...
-  -- conjure "thirty" thirty [pr (0::Int), prim "+" ((+) :: Int -> Int -> Int)]
+  -- conjure "thirty" thirty [pr (0::Int), fun "+" ((+) :: Int -> Int -> Int)]
 
-primitives :: [Prim]
-primitives  =
-  [ pr (0::Int)
-  , pr (1::Int)
-  , prim "+" ((+) :: Int -> Int -> Int)
-  , prim "*" ((*) :: Int -> Int -> Int)
-  , prim "==" ((==) :: Int -> Int -> Bool)
-  , prim "&&" (&&)
-  , prim "||" (||)
-  , prif (undefined :: Int)
+ingredients :: [Ingredient]
+ingredients  =
+  [ con (0::Int)
+  , con (1::Int)
+  , fun "+" ((+) :: Int -> Int -> Int)
+  , fun "*" ((*) :: Int -> Int -> Int)
+  , fun "==" ((==) :: Int -> Int -> Bool)
+  , fun "&&" (&&)
+  , fun "||" (||)
+  , iif (undefined :: Int)
   -- guard does not play well with usePatterns = False yet
   ]
 
