@@ -19,22 +19,22 @@ bitsum 7  =  3  -- 111
 main :: IO ()
 main  =  do
   conjure "bitsum" bitsum
-    [ pr (0 :: Int)
-    , pr (1 :: Int)
-    , prim "halve" ((`div` 2) :: Int -> Int)
-    , prim "parity" ((`mod` 2) :: Int -> Int)
-    , prim "+" ((+) :: Int -> Int -> Int)
+    [ con (0 :: Int)
+    , con (1 :: Int)
+    , fun "halve" ((`div` 2) :: Int -> Int)
+    , fun "parity" ((`mod` 2) :: Int -> Int)
+    , fun "+" ((+) :: Int -> Int -> Int)
     ]
   -- bitsum 0  =  0
   -- bitsum n  =  parity n + bitsum (halve n)
 
   conjure "bitsum" bitsum
-    [ pr (0 :: Int)
-    , pr (1 :: Int)
-    , pr (2 :: Int)
-    , prim "+" ((+) :: Int -> Int -> Int)
-    , prim "`div`" (div :: Int -> Int -> Int)
-    , prim "`mod`" (mod :: Int -> Int -> Int)
+    [ con (0 :: Int)
+    , con (1 :: Int)
+    , con (2 :: Int)
+    , fun "+" ((+) :: Int -> Int -> Int)
+    , fun "`div`" (div :: Int -> Int -> Int)
+    , fun "`mod`" (mod :: Int -> Int -> Int)
     ]
   -- bitsum 0  =  0
   -- bitsum n  =  n `mod` 2 + bitsum (n `div` 2)
