@@ -3,7 +3,6 @@
 -- Copyright (C) 2021-2025 Rudy Matela
 -- Distributed under the 3-Clause BSD licence (see the file LICENSE).
 import Conjure
-import Test.LeanCheck  -- needed for duplicatesSpec only
 
 duplicates :: [Int] -> [Int] -- Eq a => [a] -> [a]
 duplicates []  =  []
@@ -22,10 +21,10 @@ duplicates' [0,1,0,1]  =  [0,1]
 duplicates' [1,0,1,0,1]  =  [0,1]
 duplicates' [0,1,2,1]  =  [1]
 
-duplicatesSpec :: ([Int] -> [Int]) -> Bool
-duplicatesSpec duplicates  =  and
-  [ holds 360 $ \x xs -> (count (x ==) xs > 1) == elem x (duplicates xs)
-  , holds 360 $ \x xs -> count (x ==) (duplicates xs) <= 1
+duplicatesSpec :: ([Int] -> [Int]) -> [Property]
+duplicatesSpec duplicates  =
+  [ property $ \x xs -> (count (x ==) xs > 1) == elem x (duplicates xs)
+  , property $ \x xs -> count (x ==) (duplicates xs) <= 1
   ]  where  count p  =  length . filter p
 
 positionsFrom :: Int -> Int -> [Int] -> [Int]
@@ -53,32 +52,29 @@ main = do
   --                       else duplicates xs                              -- 17
   -- within reach performance wise.
   conjure "duplicates" duplicates'
-    [ pr ([] :: [Int])
-    , prim "not" not
-    , prim "&&" (&&)
-    , prim ":" ((:) :: Int -> [Int] -> [Int])
-    , prim "elem" (elem :: Int -> [Int] -> Bool)
+    [ con ([] :: [Int])
+    , fun "not" not
+    , fun "&&" (&&)
+    , fun ":" ((:) :: Int -> [Int] -> [Int])
+    , fun "elem" (elem :: Int -> [Int] -> Bool)
     , guard
     ]
 
   conjureFromSpec "duplicates" duplicatesSpec
-    [ pr ([] :: [Int])
-    , prim "not" not
-    , prim "&&" (&&)
-    , prim ":" ((:) :: Int -> [Int] -> [Int])
-    , prim "elem" (elem :: Int -> [Int] -> Bool)
+    [ con ([] :: [Int])
+    , fun "not" not
+    , fun "&&" (&&)
+    , fun ":" ((:) :: Int -> [Int] -> [Int])
+    , fun "elem" (elem :: Int -> [Int] -> Bool)
     , guard
     ]
 
   conjure "positionsFrom" positionsFrom'
-    [ pr ([] :: [Int])
-    , pr (1 :: Int)
-    , prim "+" ((+) :: Int -> Int -> Int)
-    , prim ":" ((:) :: Int -> [Int] -> [Int])
-    , prim "==" ((==) :: A -> A -> Bool)
-
---  , prif (undefined :: [Int])
-    -- cheat codes:
-    , prim "id" (id :: [Int] -> [Int])
-    , prif (undefined :: [Int] -> [Int])
+    [ con ([] :: [Int])
+    , con (1 :: Int)
+    , fun "+" ((+) :: Int -> Int -> Int)
+    , fun ":" ((:) :: Int -> [Int] -> [Int])
+    , fun "==" ((==) :: A -> A -> Bool)
+    , guard
+    , target 1080 -- 100800 -- can find after 44472 candidates in 25s
     ]
