@@ -11,30 +11,56 @@
 {-# LANGUAGE CPP, RecordWildCards, TupleSections #-}
 module Conjure.Engine
   ( conjure
-  , conjureWithMaxSize
-  , Args(..)
-  , args
-  , conjureWith
   , conjureFromSpec
-  , conjureFromSpecWith
   , conjure0
-  , conjure0With
   , Results(..)
   , conjpure
-  , conjpureWith
   , conjpureFromSpec
-  , conjpureFromSpecWith
   , conjpure0
-  , conjpure0With
   , candidateExprs
   , candidateDefns
-  , candidateDefns1
-  , candidateDefnsC
-  , conjureTheory
-  , conjureTheoryWith
+
+  -- * settings
+  , maxTests
+  , maxSize
+  , target
+
+  -- * Advanced settings
+  , maxRecursions
+  , maxEquationSize
+  , maxSearchTests
+  , maxDeconstructionSize
+  , maxConstantSize
+  , maxPatternSize
+  , maxPatternDepth
+
+  -- * Debug options
+  , showCandidates
+  , showTheory
+  , singlePattern
+  , showTests
+  , showPatterns
+  , showDeconstructions
+  , carryOn
+
+  -- * Pruning options
+  , dontRewrite
+  , dontRequireDescent
+  , omitAssortedPruning
+  , maxEarlyTests
+  , dontCopyBindings
+  , nonAtomicNumbers
+  , uniqueCandidates
+
+  -- * Properties
+  , Property
+  , property
+
+  -- * other modules
   , module Data.Express
   , module Data.Express.Fixtures
   , module Conjure.Reason
+  , module Conjure.Ingredient
   )
 where
 
@@ -45,16 +71,17 @@ import Data.Express.Fixtures hiding ((-==-))
 
 import Test.LeanCheck
 import Test.LeanCheck.Tiers
-import Test.LeanCheck.Error (errorToFalse)
+import Test.LeanCheck.Error (errorToFalse, errorToLeft)
 
 import Conjure.Expr
 import Conjure.Conjurable
-import Conjure.Prim
+import Conjure.Ingredient
 import Conjure.Defn
 import Conjure.Defn.Redundancy
 import Conjure.Defn.Test
 import Conjure.Red
 import Conjure.Reason
+import Conjure.Settings
 
 import System.CPUTime (getCPUTime)
 
@@ -72,18 +99,18 @@ import System.CPUTime (getCPUTime)
 -- > factorial 3  =  6
 -- > factorial 4  =  24
 -- >
--- > primitives :: [Prim]
--- > primitives =
--- >   [ pr (0::Int)
--- >   , pr (1::Int)
--- >   , prim "+" ((+) :: Int -> Int -> Int)
--- >   , prim "*" ((*) :: Int -> Int -> Int)
--- >   , prim "-" ((-) :: Int -> Int -> Int)
+-- > ingredients :: [Ingredient]
+-- > ingredients  =
+-- >   [ con (0::Int)
+-- >   , con (1::Int)
+-- >   , fun "+" ((+) :: Int -> Int -> Int)
+-- >   , fun "*" ((*) :: Int -> Int -> Int)
+-- >   , fun "-" ((-) :: Int -> Int -> Int)
 -- >   ]
 --
--- The conjure function does the following:
+-- The 'conjure' function does the following:
 --
--- > > conjure "factorial" factorial primitives
+-- > > conjure "factorial" factorial ingredients
 -- > factorial :: Int -> Int
 -- > -- 0.1s, testing 4 combinations of argument values
 -- > -- 0.8s, pruning with 27/65 rules
@@ -94,26 +121,28 @@ import System.CPUTime (getCPUTime)
 -- > factorial 0  =  1
 -- > factorial x  =  x * factorial (x - 1)
 --
--- The primitives list is defined with 'pr' and 'prim'.
-conjure :: Conjurable f => String -> f -> [Prim] -> IO ()
-conjure  =  conjureWith args
+-- The ingredients list is defined with 'con' and 'fun'.
+conjure :: Conjurable f => String -> f -> [Ingredient] -> IO ()
+conjure nm f  =  conjure0 nm f (const [])
 
 
 -- | Conjures an implementation from a function specification.
 --
 -- This function works like 'conjure' but instead of receiving a partial definition
--- it receives a boolean filter / property about the function.
+-- it receives a collection of test properties about the function.
 --
 -- For example, given:
 --
--- > squareSpec :: (Int -> Int) -> Bool
--- > squareSpec square  =  square 0 == 0
--- >                    && square 1 == 1
--- >                    && square 2 == 4
+-- > squarePropertySpec :: (Int -> Int) -> [Property]
+-- > squarePropertySpec square  =
+-- >   [ property $ \x -> square x >= x
+-- >   , property $ \x -> square x >= 0
+-- >   , property $ square 2 == 4
+-- >   ]
 --
 -- Then:
 --
--- > > conjureFromSpec "square" squareSpec [prim "*" ((*) :: Int -> Int -> Int)]
+-- > > conjureFromSpec "square" squareSpec [fun "*" ((*) :: Int -> Int -> Int)]
 -- > square :: Int -> Int
 -- > -- 0.1s, pruning with 2/6 rules
 -- > -- 0.1s, 1 candidates of size 1
@@ -121,212 +150,92 @@ conjure  =  conjureWith args
 -- > -- 0.1s, 1 candidates of size 3
 -- > -- 0.1s, tested 2 candidates
 -- > square x  =  x * x
---
--- This allows users to specify QuickCheck-style properties,
--- here is an example using LeanCheck:
---
--- > import Test.LeanCheck (holds, exists)
--- >
--- > squarePropertySpec :: (Int -> Int) -> Bool
--- > squarePropertySpec square  =  and
--- >   [ holds n $ \x -> square x >= x
--- >   , holds n $ \x -> square x >= 0
--- >   , exists n $ \x -> square x > x
--- >   ]  where  n = 60
-conjureFromSpec :: Conjurable f => String -> (f -> Bool) -> [Prim] -> IO ()
-conjureFromSpec  =  conjureFromSpecWith args
+conjureFromSpec :: Conjurable f => String -> (f -> [Property]) -> [Ingredient] -> IO ()
+conjureFromSpec nm p  =  conjure0 nm undefined p
 
 
 -- | Synthesizes an implementation from both a partial definition and a
 --   function specification.
 --
 --   This works like the functions 'conjure' and 'conjureFromSpec' combined.
-conjure0 :: Conjurable f => String -> f -> (f -> Bool) -> [Prim] -> IO ()
-conjure0  =  conjure0With args
-
-
--- | Like 'conjure' but allows setting the maximum size of considered expressions
---   instead of the default value of 24.
---
--- > conjureWithMaxSize 12 "function" function [...]
---
--- This function is a candidate for going away.
--- Please set maxSize in Args instead.
-conjureWithMaxSize :: Conjurable f => Int -> String -> f -> [Prim] -> IO ()
-conjureWithMaxSize sz  =  conjureWith args
-                       {  maxSize = sz
-                       ,  maxEquationSize = min sz (maxEquationSize args)
-                       }
-
-
--- | Arguments to be passed to 'conjureWith' or 'conjpureWith'.
---   See 'args' for the defaults.
-data Args = Args
-  { maxTests              :: Int  -- ^ maximum number of tests to each candidate
-  , maxSize               :: Int  -- ^ maximum size of candidate bodies
-  , target                :: Int  -- ^ enumerate further sizes of candidates until this target
-  , maxEvalRecursions     :: Int  -- ^ maximum number of recursive evaluations when testing candidates
-  , maxEquationSize       :: Int  -- ^ maximum size of equation operands
-  , maxSearchTests        :: Int  -- ^ maximum number of tests to search for defined values
-  , maxDeconstructionSize :: Int  -- ^ maximum size of deconstructions (e.g.: @_ - 1@)
-  , maxConstantSize       :: Int  -- ^ maximum size of constants (0 for no limit)
-  , maxPatternSize        :: Int  -- ^ maximum size of patterns (0 for no limit)
-  , maxPatternDepth       :: Int  -- ^ maximum depth of patterns
-
-  -- advanced & debug options --
-  , carryOn               :: Bool -- ^ whether to carry on after finding a suitable candidate
-  , showTheory            :: Bool -- ^ show theory discovered by Speculate used in pruning
-  , usePatterns           :: Bool -- ^ use pattern matching to create (recursive) candidates
-  , showRuntime           :: Bool -- ^ show runtime
-  , showCandidates        :: Bool -- ^ (debug) show candidates -- warning: wall of text
-  , showTests             :: Bool -- ^ (debug) show tests
-  , showPatterns          :: Bool -- ^ (debug) show possible LHS patterns
-  , showDeconstructions   :: Bool -- ^ (debug) show conjectured-and-allowed deconstructions
-
-  -- pruning options --
-  , rewriting             :: Bool -- ^ unique-modulo-rewriting candidates
-  , requireDescent        :: Bool -- ^ require recursive calls to deconstruct arguments
-  , adHocRedundancy       :: Bool -- ^ ad-hoc redundancy checks
-  , copyBindings          :: Bool -- ^ copy partial definition bindings in candidates
-  , earlyTests            :: Bool -- ^ perform tests early-and-independently on each binding
-  , atomicNumbers         :: Bool -- ^ restrict constant/ground numeric expressions to atoms
-  , requireZero           :: Bool -- ^ require 0 as base case for Num recursions
-  , uniqueCandidates      :: Bool -- ^ unique-modulo-testing candidates
-  }
-
-
--- | Default arguments to conjure.
---
--- * 60 tests
--- * functions of up to 24 symbols
--- * target testing over 50400 candidates
--- * maximum of one recursive call allowed in candidate bodies
--- * maximum evaluation of up to 60 recursions
--- * pruning with equations up to size 5
--- * search for defined applications for up to 100000 combinations
--- * require recursive calls to deconstruct arguments
--- * don't show the theory used in pruning
--- * do not show tested candidates
--- * do not make candidates unique module testing
-args :: Args
-args = Args
-  { maxTests               =  360
-  , maxSize                =  24
-  , target                 =  10080
-  , maxEvalRecursions      =  60
-  , maxEquationSize        =   5
-  , maxSearchTests         =  110880
-  , maxDeconstructionSize  =   4
-  , maxConstantSize        =   0 -- unlimited
-  , maxPatternSize         =   0 -- unlimited
-  , maxPatternDepth        =   1
-
-  -- advanced & debug options --
-  , carryOn                =  False
-  , showTheory             =  False
-  , usePatterns            =  True
-  , showRuntime            =  True
-  , showCandidates         =  False
-  , showTests              =  False
-  , showDeconstructions    =  False
-  , showPatterns           =  False
-
-  -- pruning options --
-  , rewriting              =  True
-  , requireDescent         =  True
-  , adHocRedundancy        =  True
-  , copyBindings           =  True
-  , earlyTests             =  True
-  , atomicNumbers          =  True
-  , requireZero            =  False
-  , uniqueCandidates       =  False
-  }
-
--- TODO: remove the requireZero option from args?
-
-
--- | Like 'conjure' but allows setting options through 'Args'/'args'.
---
--- > conjureWith args{maxSize = 18} "function" function [...]
-conjureWith :: Conjurable f => Args -> String -> f -> [Prim] -> IO ()
-conjureWith args nm f  =  conjure0With args nm f (const True)
-
--- | Like 'conjureFromSpec' but allows setting options through 'Args'/'args'.
---
--- > conjureFromSpecWith args{maxSize = 18} "function" spec [...]
-conjureFromSpecWith :: Conjurable f => Args -> String -> (f -> Bool) -> [Prim] -> IO ()
-conjureFromSpecWith args nm p  =  conjure0With args nm undefined p
-
--- | Like 'conjure0' but allows setting options through 'Args'/'args'.
-conjure0With :: Conjurable f => Args -> String -> f -> (f -> Bool) -> [Prim] -> IO ()
-conjure0With args nm f p es  =  do
+conjure0 :: Conjurable f => String -> f -> (f -> [Property]) -> [Ingredient] -> IO ()
+conjure0 nm f p es  =  do
   -- the code section below became quite ugly with time and patches.
   -- it is still maintainable and readable as it is, but perhaps
   -- needs to be cleaned up and simplified
-  t0 <- if showRuntime args
-        then getCPUTime
-        else return (-1)
+  t0 <- getCPUTime
   print (var (head $ words nm) f)
   when (length ts > 0) $ do
     putWithTimeSince t0 $ "testing " ++ show (length ts) ++ " combinations of argument values"
-    when (showTests args) $ do
+    when showTests $ do
       putStrLn $ "{-"
       putStr $ showDefn ts
       putStrLn $ "-}"
-  putWithTimeSince t0 $ "pruning with " ++ show nRules ++ "/" ++ show nREs ++ " rules"
-  when (showTheory args) $ do
-    putStrLn $ "{-"
-    printThy thy
-    putStrLn $ "-}"
-  when (not . null $ invalid thy) $ do
-    putStrLn $ "-- reasoning produced "
-            ++ show (length (invalid thy)) ++ " incorrect properties,"
-            ++ " please re-run with more tests for faster results"
-    when (showTheory args) $ do
+  if length ts == 0 && p undefined == []
+  then putStrLn $ nm ++ "  =  error \"could not reify specification, suggestion: conjureFromSpec\"\n"
+  else do
+    putWithTimeSince t0 $ "pruning with " ++ show nRules ++ "/" ++ show nREs ++ " rules"
+    when showTheory $ do
       putStrLn $ "{-"
-      putStrLn $ "invalid:"
-      putStr   $ unlines $ map showEq $ invalid thy
+      printThy thy
       putStrLn $ "-}"
-  when (showPatterns args) $ do
-    putStr $ unlines
-           $ zipWith (\i -> (("-- allowed patterns of size " ++ show i ++ "\n{-\n") ++) . (++ "-}") . unlines) [1..]
-           $ mapT showDefn
-           $ patternss results
-  when (showDeconstructions args) $ do
-    putStrLn $ "{- List of allowed deconstructions:"
-    putStr   $ unlines $ map show $ deconstructions results
-    putStrLn $ "-}"
-  pr t0 1 0 rs
+    when (not . null $ invalid thy) $ do
+      putStrLn $ "-- reasoning produced "
+              ++ show (length (invalid thy)) ++ " incorrect properties,"
+              ++ " please re-run with more tests for faster results"
+      when showTheory $ do
+        putStrLn $ "{-"
+        putStrLn $ "invalid:"
+        putStr   $ unlines $ map showEq $ invalid thy
+        putStrLn $ "-}"
+    when showPatterns $ do
+      putStr $ unlines
+             $ zipWith (\i -> (("-- allowed patterns of size " ++ show i ++ "\n{-\n") ++) . (++ "-}") . unlines) [1..]
+             $ mapT showDefn
+             $ patternss results
+    when showDeconstructions $ do
+      putStrLn $ "{- List of allowed deconstructions:"
+      putStr   $ unlines $ map show $ deconstructions results
+      putStrLn $ "-}"
+    pr t0 1 0 rs
   where
   showEq eq  =  showExpr (fst eq) ++ " == " ++ showExpr (snd eq)
   pr :: Integer -> Int -> Int -> [([Defn], [Defn])] -> IO ()
   pr t0 n t []  =  do putWithTimeSince t0 $ "tested " ++ show t ++ " candidates"
-                      putStrLn $ "cannot conjure\n"
+                      putStrLn $ nm ++ "  =  undefined  -- search exhausted\n"
   pr t0 n t ((is,cs):rs)  =  do
     let nc  =  length cs
     putWithTimeSince t0 $ show nc ++ " candidates of size " ++ show n
-    when (showCandidates args) $
+    when showCandidates $
       putStr $ unlines $ ["{-"] ++ map showDefn cs ++ ["-}"]
     case is of
       []     ->  pr t0 (n+1) (t+nc) rs
       (_:_)  ->  do pr1 t is cs
-                    when (carryOn args) $ pr t0 (n+1) (t+nc) rs
+                    when carryOn $ pr t0 (n+1) (t+nc) rs
     where
     pr1 t [] cs  =  return ()
     pr1 t (i:is) cs  =  do
       let (cs',cs'') = break (i==) cs
       let t' = t + length cs' + 1
       putWithTimeSince t0 $ "tested " ++ show t' ++ " candidates"
-      putStrLn $ showDefn i
-      when (carryOn args) $ pr1 t' is (drop 1 cs'')
+      putStrLn $ showDefn $ etaReduce $ normalizeDefn thy i
+      when carryOn $ pr1 t' is (drop 1 cs'')
   rs  =  zip iss css
-  results  =  conjpure0With args nm f p es
+  results  =  conjpure0 nm f p es
   iss  =  implementationss results
   css  =  candidatess results
   ts   =  bindings results
   thy  =  theory results
   nRules  =  length (rules thy)
   nREs  =  length (equations thy) + nRules
+  -- we could avoid the following as most are called once
+  -- but is nice to have a summary of which settings are used
+  carryOn              =  carryOnI es
+  showTests            =  showTestsI es
+  showTheory           =  showTheoryI es
+  showPatterns         =  showPatternsI es
+  showCandidates       =  showCandidatesI es
+  showDeconstructions  =  showDeconstructionsI es
 
 
 -- | Results to the 'conjpure' family of functions.
@@ -346,33 +255,19 @@ data Results = Results
 --
 -- The most important part of the result are the tiers of implementations
 -- however results also include candidates, tests and the underlying theory.
-conjpure :: Conjurable f => String -> f -> [Prim] -> Results
-conjpure =  conjpureWith args
+conjpure :: Conjurable f => String -> f -> [Ingredient] -> Results
+conjpure nm f  =  conjpure0 nm f (const [])
 
 -- | Like 'conjureFromSpec' but in the pure world.  (cf. 'conjpure')
-conjpureFromSpec :: Conjurable f => String -> (f -> Bool) -> [Prim] -> Results
-conjpureFromSpec  =  conjpureFromSpecWith args
+conjpureFromSpec :: Conjurable f => String -> (f -> [Property]) -> [Ingredient] -> Results
+conjpureFromSpec nm p  =  conjpure0 nm undefined p
 
--- | Like 'conjure0' but in the pure world.  (cf. 'conjpure')
-conjpure0 :: Conjurable f => String -> f -> (f -> Bool) -> [Prim] -> Results
-conjpure0 =  conjpure0With args
-
--- | Like 'conjpure' but allows setting options through 'Args' and 'args'.
-conjpureWith :: Conjurable f => Args -> String -> f -> [Prim] -> Results
-conjpureWith args nm f  =  conjpure0With args nm f (const True)
-
--- | Like 'conjureFromSpecWith' but in the pure world.  (cf. 'conjpure')
-conjpureFromSpecWith :: Conjurable f => Args -> String -> (f -> Bool) -> [Prim] -> Results
-conjpureFromSpecWith args nm p  =  conjpure0With args nm undefined p
-
--- | Like 'conjpure0' but allows setting options through 'Args' and 'args'.
---
--- This is where the actual implementation resides.  The functions
--- 'conjpure', 'conjpureWith', 'conjpureFromSpec', 'conjpureFromSpecWith',
--- 'conjure', 'conjureWith', 'conjureFromSpec', 'conjureFromSpecWith' and
--- 'conjure0' all refer to this.
-conjpure0With :: Conjurable f => Args -> String -> f -> (f -> Bool) -> [Prim] -> Results
-conjpure0With args@(Args{..}) nm f p es  =  Results
+-- | This is where the actual implementation resides.
+-- The functions
+-- 'conjpure', 'conjpureFromSpec', 'conjure' and 'conjureFromSpec'
+-- all refer to this.
+conjpure0 :: Conjurable f => String -> f -> (f -> [Property]) -> [Ingredient] -> Results
+conjpure0 nm f p es  =  Results
   { implementationss  =  implementationsT
   , candidatess  =  candidatesT
   , bindings  =  tests
@@ -384,33 +279,22 @@ conjpure0With args@(Args{..}) nm f p es  =  Results
   implementationsT  =  filterT implements candidatesT
   implements fx  =  defnApparentlyTerminates fx
                  && test fx
-                 && errorToFalse (p (cevl maxEvalRecursions fx))
-  candidatesT  =  (if uniqueCandidates then nubCandidates args nm f else id)
+                 && errorToFalse (testSpec maxTests $ p (cevl maxRecursions fx))
+  candidatesT  =  (if uniqueCandidates then nubCandidates maxTests maxRecursions nm f else id)
                $  (if target > 0 then targetiers target else id)
                $  (if maxSize > 0 then take maxSize else id)
                $  candidatesTT
-  (candidatesTT, thy, patternss, deconstructions)  =  candidateDefns args nm f es
+  (candidatesTT, thy, patternss, deconstructions)  =  candidateDefns nm f es
 
-  test dfn  =  all (errorToFalse . deval (conjureExpress f) maxEvalRecursions dfn False)
+  test dfn  =  all (errorToFalse . deval (conjureExpress f) maxRecursions dfn False)
             $  [funToVar lhs -==- rhs | (lhs, rhs) <- tests]
   tests  =  conjureTestDefn maxTests maxSearchTests nm f
   (-==-)  =  conjureMkEquation f
-
-
--- | Just prints the underlying theory found by "Test.Speculate"
---   without actually synthesizing a function.
-conjureTheory :: Conjurable f => String -> f -> [Prim] -> IO ()
-conjureTheory  =  conjureTheoryWith args
-
-
--- | Like 'conjureTheory' but allows setting options through 'Args'/'args'.
-conjureTheoryWith :: Conjurable f => Args -> String -> f -> [Prim] -> IO ()
-conjureTheoryWith args nm f es  =  do
-  putStrLn $ "theory with " ++ (show . length $ rules thy) ++ " rules and "
-                            ++ (show . length $ equations thy) ++ " equations"
-  printThy thy
-  where
-  Results {theory = thy}  =  conjpureWith args nm f es
+  maxTests  =  maxTestsI es
+  (target, maxSize)  =  targetAndMaxSizeI es
+  maxRecursions  =  maxRecursionsI es
+  maxSearchTests  =  maxSearchTestsI es
+  uniqueCandidates  =  uniqueCandidatesI es
 
 
 -- | Return apparently unique candidate definitions.
@@ -420,18 +304,18 @@ conjureTheoryWith args nm f es  =  do
 -- 1. tiers of candidate definitions
 -- 2. an equational theory
 -- 3. a list of allowed deconstructions
-candidateDefns :: Conjurable f => Args -> String -> f -> [Prim] -> ([[Defn]], Thy, [[Defn]], [Expr])
-candidateDefns args  =  candidateDefns' args
+candidateDefns :: Conjurable f => String -> f -> [Ingredient] -> ([[Defn]], Thy, [[Defn]], [Expr])
+candidateDefns nm f is  =  candidateDefns' nm f is
   where
-  candidateDefns'  =  if usePatterns args
-                      then candidateDefnsC
-                      else candidateDefns1
+  candidateDefns'  =  if singlePatternI is
+                      then candidateDefns1
+                      else candidateDefnsC
 
 
 -- | Return apparently unique candidate definitions
 --   where there is a single body.
-candidateDefns1 :: Conjurable f => Args -> String -> f -> [Prim] -> ([[Defn]], Thy, [[Defn]], [Expr])
-candidateDefns1 args nm f ps  =  first4 (mapT toDefn) $ candidateExprs args nm f ps
+candidateDefns1 :: Conjurable f => String -> f -> [Ingredient] -> ([[Defn]], Thy, [[Defn]], [Expr])
+candidateDefns1 nm f ps  =  first4 (mapT toDefn) $ candidateExprs nm f ps
   where
   efxs  =  conjureVarApplication nm f
   toDefn e  =  [(efxs, e)]
@@ -439,14 +323,15 @@ candidateDefns1 args nm f ps  =  first4 (mapT toDefn) $ candidateExprs args nm f
 
 
 -- | Return apparently unique candidate bodies.
-candidateExprs :: Conjurable f => Args -> String -> f -> [Prim] -> ([[Expr]], Thy, [[Defn]], [Expr])
-candidateExprs Args{..} nm f ps  =
+candidateExprs :: Conjurable f => String -> f -> [Ingredient] -> ([[Expr]], Thy, [[Defn]], [Expr])
+candidateExprs nm f is  =
   ( as \/ concatMapT (`enumerateFillings` recs) ts
   , thy
   , [[ [(efxs, eh)] ]]
   , deconstructions
   )
   where
+  ps  =  actual is  -- extract actual primitives
   es  =  map fst ps
   ts | typ efxs == boolTy  =  foldAppProducts andE [cs, rs]
                            \/ foldAppProducts orE  [cs, rs]
@@ -463,7 +348,7 @@ candidateExprs Args{..} nm f ps  =
   eh  =  holeAsTypeOf efxs
   efxs  =  conjureVarApplication nm f
   (ef:exs)  =  unfoldApp efxs
-  keep | rewriting  =  isRootNormalC thy . fastMostGeneralVariation
+  keep | rewrite    =  isRootNormalC thy . fastMostGeneralVariation
        | otherwise  =  const True
   keepR | requireDescent  =  descends isDecOf efxs
         | otherwise       =  const True
@@ -487,15 +372,21 @@ candidateExprs Args{..} nm f ps  =
         $  foldAppProducts ef [forN h | h <- conjureArgumentHoles f]
   thy  =  doubleCheck (===)
        .  theoryFromAtoms (===) maxEquationSize . (:[]) . nub
-       $  cjHoles (prim nm f:ps) ++ [val False, val True] ++ es
-  (===)  =  cjAreEqual (prim nm f:ps) maxTests
+       $  cjHoles (fun nm f:ps) ++ [val False, val True] ++ es
+  (===)  =  cjAreEqual (fun nm f:ps) maxTests
+  maxTests               =  maxTestsI is
+  maxEquationSize        =  maxEquationSizeI is
+  maxDeconstructionSize  =  maxDeconstructionSizeI is
+  requireDescent         =  requireDescentI is
+  rewrite                =  rewriteI is
 
 
 -- | Return apparently unique candidate definitions
 --   using pattern matching.
-candidateDefnsC :: Conjurable f => Args -> String -> f -> [Prim] -> ([[Defn]], Thy, [[Defn]], [Expr])
-candidateDefnsC Args{..} nm f ps  =
-  ( discardT hasRedundant $ concatMapT fillingsFor fss
+candidateDefnsC :: Conjurable f => String -> f -> [Ingredient] -> ([[Defn]], Thy, [[Defn]], [Expr])
+candidateDefnsC nm f is =
+  ( discardT hasRedundant $ catconMapT fillingsFor partialDefns
+  -- above we catconMapT to prefer smaller recursive calls
   , thy
   , mapT (map (,eh)) pats
   , deconstructions
@@ -503,11 +394,12 @@ candidateDefnsC Args{..} nm f ps  =
   where
   pats | maxPatternSize > 0  =  take maxPatternSize $ conjurePats maxPatternDepth es nm f
        | otherwise           =                        conjurePats maxPatternDepth es nm f
-  fss  =  concatMapT ps2fss pats
+  partialDefns  =  concatMapT partialDefnsFromPats pats
   -- replaces the any guard symbol with a guard of the correct type
+  ps  =  actual is  -- extract actual ingredients/primitives from the list
   es  =  [if isGuardSymbol e then conjureGuard f else e | (e,_) <- ps]
 
-  eh  =  holeAsTypeOf efxs
+  eh  =  conjureResultHole f
   efxs  =  conjureVarApplication nm f
   (ef:_)  =  unfoldApp efxs
 
@@ -540,44 +432,57 @@ candidateDefnsC Args{..} nm f ps  =
   isNumeric  =  conjureIsNumeric f
 
   (-==-)  =  conjureMkEquation f
+  etests  =  map (first (mappArgs exprExpr)) tests
   tests  =  conjureTestDefn maxTests maxSearchTests nm f
   exprExpr  =  conjureExpress f
 
-  ps2fss :: [Expr] -> [[Defn]]
-  ps2fss pats  =  discardT isRedundant
-               .  products  -- alt: use delayedProducts
-               $  map p2eess pats
+  distritests :: [(Expr,Expr)] -> [Expr] -> [([(Expr,Expr)],Expr)]
+  distritests _ []  =  []
+  distritests ts (pat:pats)  =  (ys, pat) : distritests ns pats
+    where
+    (ys, ns)  =  partition (\(lhs,_) -> lhs `isInstanceOf` pat) ts
+
+  partialDefnsFromPats :: [Expr] -> [[Defn]]
+  partialDefnsFromPats pats  =  discardT isRedundant
+                             .  products  -- alt: use delayedProducts
+                             .  map (uncurry bindingsForPattern)
+                             .  distritests etests
+                             $  pats
     -- delayedProducts makes the number of patterns counts as the size+1.
     where
-    p2eess :: Expr -> [[Bndn]]
+    bindingsForPattern :: [(Expr,Expr)] -> Expr -> [[Bndn]]
     -- the following guarded line is an optional optimization
     -- if the function is defined for the given pattern,
     -- simply use its return value as the only possible result
-    p2eess pat | copyBindings && isGroundPat f pat  =  [[(pat, toValPat f pat)]]
-    p2eess pat  =  mapT (pat,)
-                .  filterT keepBase
-                .  appsWith pat
-                .  drop 1 -- this excludes the function name itself
-                $  vars pat ++ [eh | any (uncurry should) (zip aess aes)]
+    bindingsForPattern ts pat
+      | copyBindings && isGroundPat f pat  =  [[(pat, toValPat f pat)]]
+      | otherwise  =  mapT (pat,)
+                   .  filterT keepB
+                   .  appsWith pat
+                   .  drop 1 -- this excludes the function name itself
+                   $  vars pat ++ [eh | any (uncurry should) (zip aess aes)]
       where
-      keepBase
-        | not earlyTests  =  const True
-        | all isVar (unfoldApp pat)  =  const True
-        | otherwise  =  \e -> hasHole e || reallyKeepBase e
-      reallyKeepBase e  =  and
-        [ errorToFalse $ eval False $ (e //- bs) -==- rhs
-        | (lhs,rhs) <- tests
-        -- filter test bindings that match the current pattern:
-        , Just bs <- [lhs `matchArgs` pat]
+      keepB
+        | maxEarlyTests <= 0  =  const True
+        | length pats < 2  =  const True  -- just one pat, test later
+        | otherwise  =  \e -> isNumeric eh && hasHole e || reallyKeepB e
+      reallyKeepB e  =  and
+        [ errholeToTrue $ eval False $ (e //- bs) -==- rhs
+        | (lhs,rhs) <- take maxEarlyTests ts
+        , Just bs <- [lhs `match` pat]  -- always should match
         ]
-      matchArgs efxs efys  =  fold (map exprExpr (drop 1 (unfoldApp efxs)))
-                      `match` fold               (drop 1 (unfoldApp efys))
 
-      -- computes whether we should include a recurse for this given argument
-      -- numeric arguments additionally require 0 to be present as a case
-      -- for recursion
-      should aes ae  =  length (nub aes) > 1 && hasVar ae && (isApp ae || isUnbreakable ae)
-                     && (not requireZero || not (isNumeric ae) || any isZero aes)
+      -- computes whether we should include a recurse for this given argument:
+      -- 1. more than one LHS pattern overall
+      -- 2. there should be at least a variable
+      -- 3. it should either:
+      --    * we do not require descent
+      --    * be a breakdown such as _:_ or Tree _ _ _
+      --    * or be of an unbreakable/atomic type such as (_ :: Int)
+      --      in the presence of deconstructions
+      should aes ae  =  length (nub aes) > 1
+                     && hasVar ae
+                     && (not requireDescent || isApp ae || isUnbreakable ae && notNull deconstructions)
       aes   =                  (tail . unfoldApp . rehole) pat
       aess  =  transpose $ map (tail . unfoldApp . rehole) pats
 
@@ -644,10 +549,24 @@ candidateDefnsC Args{..} nm f ps  =
 
   thy  =  doubleCheck (===)
        .  theoryFromAtoms (===) maxEquationSize . (:[]) . nub
-       $  cjHoles (prim nm f:ps) ++ [val False, val True] ++ es
-  (===)  =  cjAreEqual (prim nm f:ps) maxTests
+       $  cjHoles (fun nm f:ps) ++ [val False, val True] ++ es
+  (===)  =  cjAreEqual (fun nm f:ps) maxTests
   isUnbreakable  =  conjureIsUnbreakable f
 
+  maxTests               =  maxTestsI is
+  maxSearchTests         =  maxSearchTestsI is
+  maxEquationSize        =  maxEquationSizeI is
+  maxConstantSize        =  maxConstantSizeI is
+  maxDeconstructionSize  =  maxDeconstructionSizeI is
+  maxPatternDepth        =  maxPatternDepthI is
+  maxPatternSize         =  maxPatternSizeI is
+  requireDescent         =  requireDescentI is
+  maxEarlyTests          =  maxEarlyTestsI is
+  copyBindings           =  copyBindingsI is
+  adHocRedundancy        =  assortedPruningI is -- TODO: rename
+  atomicNumbers          =  atomicNumbersI is
+  rewriting              =  rewriteI is
+
 
 -- | Checks if the given pattern is a ground pattern.
 --
@@ -706,9 +625,9 @@ keepIf _  =  error "Conjure.Engine.keepIf: not an if"
 
 -- equality between candidates
 
-nubCandidates :: Conjurable f => Args -> String -> f -> [[Defn]] -> [[Defn]]
-nubCandidates Args{..} nm f  =
-  discardLaterT $ equalModuloTesting maxTests maxEvalRecursions nm f
+nubCandidates :: Conjurable f => Int -> Int -> String -> f -> [[Defn]] -> [[Defn]]
+nubCandidates maxTests maxRecursions nm f  =
+  discardLaterT $ equalModuloTesting maxTests maxRecursions nm f
 
 
 --- tiers utils ---
@@ -752,5 +671,52 @@ targetiers n xss
   | otherwise  =  case xss of [] -> []
                               (xs:xss) -> xs : targetiers (n - length xs) xss
 
+normalizeDefn :: Thy -> Defn -> Defn
+normalizeDefn  =  map . normalizeBndn
+
+-- This function is quite expensive to run with bad complexity,
+-- but is fine on Conjure: candidates are at most a few dozen symbols long
+-- and this function just runs once.
+normalizeBndn :: Thy -> Bndn -> Bndn
+normalizeBndn thy (lhs, rhs)
+  | ef `elem` vars rhs  =  (lhs, mapInnerFirstOuterLast commutsort rhs)
+  | otherwise           =  (lhs, rhs)
+  where
+  ef:_  =  unfoldApp lhs
+  -- recursive calls come later in this ordering
+  commutsort (eo :$ ex :$ ey)
+    | isCommutative thy eo  =  if (ef `elem` vars ex, ex)
+                               <= (ef `elem` vars ey, ey)
+                               then eo :$ ex :$ ey
+                               else eo :$ ey :$ ex
+  commutsort e  =  e
+
 boolTy :: TypeRep
 boolTy  =  typ b_
+
+-- TODO: move these property things to a module of their own?
+
+-- | A test property provided as part of a specification for 'conjureFromSpec'.
+--
+-- Construct with 'property'.
+type Property  =  [Bool]
+
+-- | Provides a single test property to 'conjureFromSpec'.
+property :: Testable a => a -> Property
+property  =  map snd . results
+
+testSpec :: Int -> [Property] -> Bool
+testSpec maxTests  =  and . map (and . take maxTests)
+
+-- like errorToFalse, but returns True upon finding a placeholder hole
+errholeToTrue :: Bool -> Bool
+errholeToTrue p  =  case errorToLeft p of
+                    Right q -> q
+                    Left "conjureResultHole: placeholder for recursive call?" -> True
+                    Left _ -> False
+
+
+catconMapT :: (a -> [[b]]) -> [[a]] -> [[b]]
+catconMapT f  =  foldr (\+:/) [] . map (foldr (\/) []) . mapT f
+  where
+  xss \+:/ yss  =  ([]:yss) \/ xss
