@@ -39,6 +39,9 @@ module Conjure.Expr
   , isGuardSymbol
   , isGuard
   , hasGuard
+  , mapInnerFirstOuterLast
+  , mappArgs
+  , mappFun
 
   , enumerateAppsFor
   , enumerateFillings
@@ -69,7 +72,7 @@ import Data.Express.Fixtures hiding ((-==-))
 import Data.Dynamic
 -- import Control.Applicative ((<$>)) -- for GHC <= 7.8
 
-import Test.LeanCheck (mapT, filterT, (\/), delay, productWith)
+import Test.LeanCheck (mapT, filterT, (\/), delay, productWith, concatMapT)
 import Test.LeanCheck.Tiers (products)
 import Test.LeanCheck.Utils.Types (A, B, C, D, E, F)
 
@@ -352,14 +355,15 @@ possibleHoles  =  nubSort . ph . nubSort . map holeAsTypeOf
 
 -- -- Expression enumeration -- --
 
--- | Enumerate applications between values of the given list of primitives
---   and of the given expressions's type.
+-- | Enumerate applications between values of
+--   the given list of atomic expressions
+--   and of the given resulting expressions's type.
 --
 -- __Arguments:__
 --
 -- 1. an 'Expr' whose type we are interested in
 -- 2. a filtering function, returning 'True' for the expressions to keep
--- 3. a list of primitives to be used in building expression.
+-- 3. a list of atomic expressions to be used in building expression.
 --
 -- __Result:__ a potentially infinite list of list of enumerated expressions
 --
@@ -373,11 +377,15 @@ enumerateAppsFor h keep es  =  for h
   for h  =  filter (\e -> typ h == typ e) es : apps
     where
     apps  =  foldr (\/) []
-          [  filterT keep $ fliproductWith (:$) (ufor hf) (ufor hx)
+          [  filterT keep $ concatMapT afor (ufor hf)
           |  hf <- hs
           ,  hx <- hs
           ,  Just hfx <- [hf $$ hx]
           ,  typ h == typ hfx
+          -- the following allows chains of guards:
+          , let afor ef  =  mapT (ef :$) $ if isGuard (ef :$ hx)
+                                           then for hx
+                                           else ufor hx
           ]
   -- unguarded for
   ufor | any isGuardSymbol es  =  filterT (not . isGuard) . for
@@ -629,6 +637,30 @@ isStrictSubexprOf :: Expr -> Expr -> Bool
 isStrictSubexprOf e1 e2  =  e1 /= e2
                          && e1 `isSubexprOf` e2
 
+-- TODO: move mapInner to Express?
+--       of course with a shorter name.
+--
+-- The long name here is intentional to prevent a future clash.  :-)
+mapInnerFirstOuterLast :: (Expr -> Expr) -> Expr -> Expr
+mapInnerFirstOuterLast f  =  m
+  where
+  m (ef :$ ex)  =  f (m ef :$ m ex)
+  m e  =  f e
+
+-- TODO: move mappArgs to Express?
+mappArgs :: (Expr -> Expr) -> Expr -> Expr
+mappArgs f  =  m
+  where
+  m (ef :$ ex)  =  m ef :$ f ex
+  m e  =  e
+
+-- TODO: move mappFun to Express?
+mappFun :: (Expr -> Expr) -> Expr -> Expr
+mappFun f  =  m
+  where
+  m (ef :$ ex)  =  m ef :$ ex
+  m ef  =  f ef
+
 instance Express A where  expr  =  val
 instance Express B where  expr  =  val
 instance Express C where  expr  =  val
