@@ -26,8 +26,8 @@ module Conjure.Utils
 #endif
   , idIO
   , mapHead
-  , sets
   , headOr
+  , notNull
   , allEqual
   , allEqualOn
   , allEqual2
@@ -137,20 +137,18 @@ mapHead :: (a -> a) -> [a] -> [a]
 mapHead f (x:xs)  =  f x : xs
 mapHead f []  =  error "Conjure.Utils.mapHead: empty list"
 
--- | Return sets of values based on the list.
---
--- The values in the list must me unique.
-sets :: [a] -> [[a]]
-sets []  =  [[]]
-sets (x:xs)  =  map (x:) ss ++ ss
-  where
-  ss  =  sets xs
-
 -- | Like 'head' but allows providing a default value.
 headOr :: a -> [a] -> a
 headOr x []  =  x
 headOr _ (x:xs)  =  x
 
+-- | Is the given list not null?
+--
+-- Equivalent to @not . null@
+notNull :: [a] -> Bool
+notNull []  =  False
+notNull (_:_)  =  True
+
 -- | Lists choices of values.
 choices :: [a] -> [(a,[a])]
 choices []  =  []
